@@ -24,14 +24,15 @@ this module can be modified.
 from __future__ import division, print_function, absolute_import
 
 import os
-import inspect
 import warnings
 import threading
+import contextlib
 from enum import Enum, IntEnum, unique
 
 import attr
 
-from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
+from hypothesis.errors import InvalidState, InvalidArgument, \
+    HypothesisDeprecationWarning
 from hypothesis.internal.compat import text_type
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.internal.reflection import proxies, \
@@ -39,6 +40,9 @@ from hypothesis.internal.reflection import proxies, \
 from hypothesis.internal.validation import try_convert
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
+if False:
+    from typing import Any, Dict, List  # noqa
+
 __all__ = [
     'settings',
 ]
@@ -47,7 +51,7 @@ __all__ = [
 unlimited = UniqueIdentifier('unlimited')
 
 
-all_settings = {}
+all_settings = {}  # type: Dict[str, Setting]
 
 
 class settingsProperty(object):
@@ -106,15 +110,28 @@ class settingsMeta(type):
             assert default_variable.value is not None
         return default_variable.value
 
-    @default.setter
-    def default(self, value):
-        raise AttributeError('Cannot assign settings.default')
-
     def _assign_default_internal(self, value):
         default_variable.value = value
 
+    def __setattr__(self, name, value):
+        if name == 'default':
+            raise AttributeError(
+                'Cannot assign to the property settings.default - '
+                'consider using settings.load_profile instead.'
+            )
+        elif not (isinstance(value, settingsProperty) or name.startswith('_')):
+            raise AttributeError(
+                'Cannot assign hypothesis.settings.%s=%r - the settings '
+                'class is immutable.  You can change the global default '
+                'settings with settings.load_profile, or use @settings(...) '
+                'to decorate your test instead.' % (name, value)
+            )
+        return type.__setattr__(self, name, value)
 
-class settings(settingsMeta('settings', (object,), {})):
+
+class settings(
+    settingsMeta('settings', (object,), {})  # type: ignore
+):
     """A settings object controls a variety of parameters that are used in
     falsification. These may control both the falsification strategy and the
     details of the data that is generated.
@@ -127,7 +144,7 @@ class settings(settingsMeta('settings', (object,), {})):
         '_construction_complete', 'storage'
     ]
     __definitions_are_locked = False
-    _profiles = {}
+    _profiles = {}  # type: dict
 
     def __getattr__(self, name):
         if name in all_settings:
@@ -136,6 +153,7 @@ class settings(settingsMeta('settings', (object,), {})):
             raise AttributeError('settings has no attribute %s' % (name,))
 
     def __init__(self, parent=None, **kwargs):
+        # type: (settings, **Any) -> None
         if (
             kwargs.get('database', not_set) is not_set and
             kwargs.get('database_file', not_set) is not not_set
@@ -147,6 +165,11 @@ class settings(settingsMeta('settings', (object,), {})):
                 kwargs['database'] = ExampleDatabase(kwargs['database_file'])
         if not kwargs.get('perform_health_check', True):
             kwargs['suppress_health_check'] = HealthCheck.all()
+        if kwargs.get('max_shrinks') == 0:
+            kwargs['phases'] = tuple(
+                p for p in _validate_phases(kwargs.get('phases'))
+                if p != Phase.shrink
+            )
         self._construction_complete = False
         deprecations = []
         defaults = parent or settings.default
@@ -239,7 +262,7 @@ class settings(settingsMeta('settings', (object,), {})):
         return test_to_use
 
     @classmethod
-    def define_setting(
+    def _define_setting(
         cls, name, description, default, options=None,
         validator=None, show_default=True, future_default=not_set,
         deprecation_message=None, hide_repr=not_set,
@@ -254,7 +277,6 @@ class settings(settingsMeta('settings', (object,), {})):
           the first time it is accessed on any given settings object.
         """
         if settings.__definitions_are_locked:
-            from hypothesis.errors import InvalidState
             raise InvalidState(
                 'settings have been locked and may no longer be defined.'
             )
@@ -306,11 +328,25 @@ class settings(settingsMeta('settings', (object,), {})):
         bits = []
         for name, setting in all_settings.items():
             value = getattr(self, name)
+            # The only settings that are not shown are those that are
+            # deprecated and left at their default values.
             if value != setting.default or not setting.hide_repr:
                 bits.append('%s=%r' % (name, value))
         return 'settings(%s)' % ', '.join(sorted(bits))
 
+    def show_changed(self):
+        bits = []
+        for name, setting in all_settings.items():
+            value = getattr(self, name)
+            if value != setting.default:
+                bits.append('%s=%r' % (name, value))
+        return ', '.join(sorted(bits, key=len))
+
     def __enter__(self):
+        note_deprecation(
+            'Settings should be determined only by global state or with the '
+            '@settings decorator.'
+        )
         default_context_manager = default_variable.with_value(self)
         self.defaults_stack().append(default_context_manager)
         default_context_manager.__enter__()
@@ -322,6 +358,7 @@ class settings(settingsMeta('settings', (object,), {})):
 
     @staticmethod
     def register_profile(name, parent=None, **kwargs):
+        # type: (str, settings, **Any) -> None
         """Registers a collection of values to be used as a settings profile.
 
         Settings profiles can be loaded by name - for example, you might
@@ -350,6 +387,7 @@ class settings(settingsMeta('settings', (object,), {})):
 
     @staticmethod
     def get_profile(name):
+        # type: (str) -> settings
         """Return the profile with the given name."""
         if not isinstance(name, (str, text_type)):
             note_deprecation('name=%r must be a string' % (name,))
@@ -360,6 +398,7 @@ class settings(settingsMeta('settings', (object,), {})):
 
     @staticmethod
     def load_profile(name):
+        # type: (str) -> None
         """Loads in the settings defined in the profile provided.
 
         If the profile does not exist, InvalidArgument will be raised.
@@ -372,6 +411,13 @@ class settings(settingsMeta('settings', (object,), {})):
         settings._assign_default_internal(settings.get_profile(name))
 
 
+@contextlib.contextmanager
+def local_settings(s):
+    default_context_manager = default_variable.with_value(s)
+    with default_context_manager:
+        yield s
+
+
 @attr.s()
 class Setting(object):
     name = attr.ib()
@@ -384,7 +430,7 @@ class Setting(object):
     hide_repr = attr.ib()
 
 
-settings.define_setting(
+settings._define_setting(
     'min_satisfying_examples',
     default=not_set,
     description="""
@@ -397,7 +443,7 @@ max_examples setting.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'max_examples',
     default=100,
     description="""
@@ -406,7 +452,7 @@ counter-example, falsification will terminate.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'max_iterations',
     default=not_set,
     description="""
@@ -418,7 +464,7 @@ useful for this purpose than a user setting.  It no longer has any effect.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'buffer_size',
     default=8 * 1024,
     description="""
@@ -429,13 +475,16 @@ your tests slower.
 )
 
 
-settings.define_setting(
+settings._define_setting(
     'max_shrinks',
-    default=500,
+    default=not_set,
     description="""
-Once this many successful shrinks have been performed, Hypothesis will assume
-something has gone a bit wrong and give up rather than continuing to try to
-shrink the example.
+Passing ``max_shrinks=0`` disables the shrinking phase (see the ``phases``
+setting), but any other value has no effect and uses a general heuristic.
+""",
+    deprecation_message="""
+The max_shrinks setting has been disabled, as internal heuristics are more
+useful for this purpose than a user setting.
 """
 )
 
@@ -447,7 +496,7 @@ def _validate_timeout(n):
         return n
 
 
-settings.define_setting(
+settings._define_setting(
     'timeout',
     default=60,
     description="""
@@ -468,7 +517,7 @@ setting has gone away).
     validator=_validate_timeout
 )
 
-settings.define_setting(
+settings._define_setting(
     'derandomize',
     default=False,
     description="""
@@ -482,7 +531,7 @@ find novel breakages.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'strict',
     default=os.getenv('HYPOTHESIS_STRICT_MODE') == 'true',
     description="""
@@ -499,11 +548,11 @@ warnings.simplefilter('error', HypothesisDeprecationWarning).
 )
 
 
-def _validate_database(db, __from_db_file=False):
+def _validate_database(db, _from_db_file=False):
     from hypothesis.database import ExampleDatabase
     if db is None or isinstance(db, ExampleDatabase):
         return db
-    if __from_db_file or db is not_set:
+    if _from_db_file or db is not_set:
         return ExampleDatabase(db)
     raise InvalidArgument(
         'Arguments to the database setting must be None or an instance of '
@@ -513,7 +562,7 @@ def _validate_database(db, __from_db_file=False):
     )
 
 
-settings.define_setting(
+settings._define_setting(
     'database',
     default=not_set,
     show_default=False,
@@ -526,7 +575,7 @@ database, or any path for a directory-based example database.
     validator=_validate_database,
 )
 
-settings.define_setting(
+settings._define_setting(
     'database_file',
     default=not_set,
     show_default=False,
@@ -534,7 +583,7 @@ settings.define_setting(
 The file or directory location to save and load previously tried examples;
 `:memory:` for an in-memory cache or None to disable caching entirely.
 """,
-    validator=lambda f: _validate_database(f, __from_db_file=True),
+    validator=lambda f: _validate_database(f, _from_db_file=True),
     deprecation_message="""
 The `database_file` setting is deprecated in favor of the `database`
 setting, and will be removed in a future version.  It only exists at
@@ -564,6 +613,7 @@ class HealthCheck(Enum):
 
     @classmethod
     def all(cls):
+        # type: () -> List[HealthCheck]
         bad = (HealthCheck.exception_in_generation, HealthCheck.random_module)
         return [h for h in list(cls) if h not in bad]
 
@@ -619,6 +669,7 @@ class Verbosity(IntEnum):
 
     @staticmethod
     def _get_default():
+        # type: () -> Verbosity
         var = os.getenv('HYPOTHESIS_VERBOSITY_LEVEL')
         if var is not None:  # pragma: no cover
             try:
@@ -639,7 +690,7 @@ class Verbosity(IntEnum):
         return 'Verbosity.%s' % (self.name,)
 
 
-settings.define_setting(
+settings._define_setting(
     'verbosity',
     options=tuple(Verbosity),
     default=Verbosity._get_default(),
@@ -657,7 +708,7 @@ def _validate_phases(phases):
     return phases
 
 
-settings.define_setting(
+settings._define_setting(
     'phases',
     default=tuple(Phase),
     description=(
@@ -667,7 +718,7 @@ settings.define_setting(
     validator=_validate_phases,
 )
 
-settings.define_setting(
+settings._define_setting(
     name='stateful_step_count',
     default=50,
     description="""
@@ -675,7 +726,7 @@ Number of steps to run a stateful program for before giving up on it breaking.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'perform_health_check',
     default=not_set,
     description=u"""
@@ -710,14 +761,14 @@ def validate_health_check_suppressions(suppressions):
     return suppressions
 
 
-settings.define_setting(
+settings._define_setting(
     'suppress_health_check',
     default=(),
     description="""A list of health checks to disable.""",
     validator=validate_health_check_suppressions
 )
 
-settings.define_setting(
+settings._define_setting(
     'deadline',
     default=not_set,
     description=u"""
@@ -736,17 +787,15 @@ deadline and have not explicitly set a deadline yourself.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'use_coverage',
-    default=True,
+    default=not_set,
+    deprecation_message="""
+use_coverage no longer does anything and can be removed from your settings.
+""",
     description="""
-Whether to use coverage information to improve Hypothesis's ability to find
-bugs.
-
-You should generally leave this turned on unless your code performs
-poorly when run under coverage. If you turn it off, please file a bug report
-or add a comment to an existing one about the problem that prompted you to do
-so.
+A flag to enable a feature that no longer exists. This setting is present
+only for backwards compatibility purposes.
 """
 )
 
@@ -762,17 +811,18 @@ class PrintSettings(Enum):
     """Make an educated guess as to whether it would be appropriate to print
     the blob.
 
-    The current rules are that this will print if both:
+    The current rules are that this will print if:
 
     1. The output from Hypothesis appears to be unsuitable for use with
-       :func:`~hypothesis.example`.
-    2. The output is not too long."""
+       :func:`~hypothesis.example`, and
+    2. The output is not too long, and
+    3. Verbosity is at least normal."""
 
     ALWAYS = 2
     """Always print a blob on failure."""
 
 
-settings.define_setting(
+settings._define_setting(
     'print_blob',
     default=PrintSettings.INFER,
     description="""
@@ -788,6 +838,7 @@ settings.lock_further_definitions()
 
 
 def note_deprecation(message, s=None):
+    # type: (str, settings) -> None
     if s is None:
         s = settings.default
     assert s is not None
