@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -20,14 +20,15 @@ from __future__ import division, print_function, absolute_import
 import os
 import sys
 
-import coverage
-
 import hypothesis
 from hypothesis.errors import StopTest, DeadlineExceeded, \
     HypothesisException, UnsatisfiedAssumption
 from hypothesis.internal.compat import text_type, binary_type, \
     encoded_filepath
 
+if False:
+    from typing import Dict  # noqa
+
 
 def belongs_to(package):
     root = os.path.dirname(package.__file__)
@@ -50,11 +51,10 @@ def belongs_to(package):
 
 PREVENT_ESCALATION = os.getenv('HYPOTHESIS_DO_NOT_ESCALATE') == 'true'
 
-FILE_CACHE = {}
+FILE_CACHE = {}  # type: Dict[bytes, bool]
 
 
 is_hypothesis_file = belongs_to(hypothesis)
-is_coverage_file = belongs_to(coverage)
 
 HYPOTHESIS_CONTROL_EXCEPTIONS = (
     DeadlineExceeded, StopTest, UnsatisfiedAssumption
@@ -78,8 +78,3 @@ def escalate_hypothesis_internal_error():
         e, (HypothesisException,) + HYPOTHESIS_CONTROL_EXCEPTIONS,
     ):
         raise
-    # This is so that if we do something wrong and trigger an internal Coverage
-    # error we don't try to catch it. It should be impossible to trigger, but
-    # you never know.
-    if is_coverage_file(filepath):  # pragma: no cover
-        raise
